@@ -4,6 +4,7 @@ from .. import util
 from .. import visual
 
 from ..base import Trimesh
+from ..parent import Geometry
 from ..points import PointCloud
 from ..scene.scene import Scene, append_scenes
 from ..constants import log_time, log
@@ -11,13 +12,17 @@ from ..constants import log_time, log
 from . import misc
 from .ply import _ply_loaders
 from .stl import _stl_loaders
+from .dae import _collada_loaders
+from .obj import _obj_loaders
+from .off import _off_loaders
 from .misc import _misc_loaders
 from .gltf import _gltf_loaders
 from .assimp import _assimp_loaders
 from .threemf import _three_loaders
 from .openctm import _ctm_loaders
-from .wavefront import _obj_loaders
 from .xml_based import _xml_loaders
+from .binvox import _binvox_loaders
+from .xyz import _xyz_loaders
 
 
 try:
@@ -34,7 +39,8 @@ except BaseException as E:
 
         Raises
         ----------
-        path_exception : Whatever failed when we imported path
+        path_exception : BaseException
+          Whatever failed when we imported path
         """
         raise _path_exception
 
@@ -68,7 +74,6 @@ def available_formats():
     loaders = mesh_formats()
     loaders.extend(path_formats())
     loaders.extend(compressed_loaders.keys())
-
     return loaders
 
 
@@ -77,18 +82,18 @@ def load(file_obj,
          resolver=None,
          **kwargs):
     """
-    Load a mesh or vectorized path into objects:
+    Load a mesh or vectorized path into objects like
     Trimesh, Path2D, Path3D, Scene
 
     Parameters
-    ---------
+    -----------
     file_obj : str, or file- like object
       The source of the data to be loadeded
     file_type: str
       What kind of file type do we have (eg: 'stl')
     resolver : trimesh.visual.Resolver
       Object to load referenced assets like materials and textures
-    kwargs : **
+    kwargs : dict
       Passed to geometry __init__
 
     Returns
@@ -97,12 +102,9 @@ def load(file_obj,
       Loaded geometry as trimesh classes
     """
     # check to see if we're trying to load something
-    # that is already a native trimesh object
-    # do the check by name to avoid circular imports
-    out_types = ('Trimesh', 'Path', 'Scene')
-    if any(util.is_instance_named(file_obj, t)
-           for t in out_types):
-        log.info('Loaded called on %s object, returning input',
+    # that is already a native trimesh Geometry subclass
+    if isinstance(file_obj, Geometry):
+        log.info('Load called on %s object, returning input',
                  file_obj.__class__.__name__)
         return file_obj
 
@@ -137,13 +139,19 @@ def load(file_obj,
             loaded = load_compressed(file_obj,
                                      file_type=file_type,
                                      **kwargs)
+        elif file_type in voxel_loaders:
+            loaded = voxel_loaders[file_type](
+                file_obj,
+                file_type=file_type,
+                resolver=resolver,
+                **kwargs)
         else:
             if file_type in ['svg', 'dxf']:
                 # call the dummy function to raise the import error
                 # this prevents the exception from being super opaque
                 load_path()
             else:
-                raise ValueError('File type: %s not supported',
+                raise ValueError('File type: %s not supported' %
                                  file_type)
     finally:
         # close any opened files even if we crashed out
@@ -171,16 +179,18 @@ def load_mesh(file_obj,
     Load a mesh file into a Trimesh object
 
     Parameters
-    ---------
-    file_obj:  str or file-like object
-    file_type: str representing file type (eg: 'stl')
-    kwargs:    passed to Trimesh constructor
+    -----------
+    file_obj : str or file object
+      File name or file with mesh data
+    file_type : str or None
+      Which file type, e.g. 'stl'
+    kwargs : dict
+      Passed to Trimesh constructor
 
-    Returns:
+    Returns
     ----------
-    mesh: Trimesh object, or a list of Trimesh objects
-          depending on the file format.
-
+    mesh : trimesh.Trimesh or trimesh.Scene
+      Loaded geometry data
     """
 
     # parse the file arguments into clean loadable form
@@ -204,9 +214,6 @@ def load_mesh(file_obj,
         if util.is_file(file_obj):
             file_obj.close()
 
-        log.debug('loaded mesh using %s',
-                  mesh_loaders[file_type].__name__)
-
         if not isinstance(results, list):
             results = [results]
 
@@ -217,6 +224,10 @@ def load_mesh(file_obj,
             loaded[-1].metadata.update(metadata)
         if len(loaded) == 1:
             loaded = loaded[0]
+        # show the repr for loaded
+        log.debug('loaded {} using {}'.format(
+            str(loaded),
+            mesh_loaders[file_type].__name__))
     finally:
         # if we failed to load close file
         if opened:
@@ -383,24 +394,23 @@ def load_kwargs(*args, **kwargs):
             scene.graph.base_frame = kwargs['base_frame']
         if 'metadata' in kwargs:
             scene.metadata.update(kwargs['metadata'])
-
         return scene
 
-    def handle_trimesh_kwargs():
+    def handle_mesh():
         """
-        Load information with vertices and faces into a mesh
-        or PointCloud object.
+        Handle the keyword arguments for a Trimesh object
         """
+        # if they've been serialized as a dict
         if (isinstance(kwargs['vertices'], dict) or
                 isinstance(kwargs['faces'], dict)):
             return Trimesh(**misc.load_dict(kwargs))
-        elif kwargs['faces'] is None:
-            # vertices without faces returns a PointCloud
-            return PointCloud(**kwargs)
-        else:
-            return Trimesh(**kwargs)
+        # otherwise just load that puppy
+        return Trimesh(**kwargs)
 
-    def handle_trimesh_export():
+    def handle_export():
+        """
+        Handle an exported mesh.
+        """
         data, file_type = kwargs['data'], kwargs['file_type']
         if not isinstance(data, dict):
             data = util.wrap_as_stream(data)
@@ -408,6 +418,9 @@ def load_kwargs(*args, **kwargs):
                                     file_type=file_type)
         return Trimesh(**k)
 
+    def handle_pointcloud():
+        return PointCloud(**kwargs)
+
     # if we've been passed a single dict instead of kwargs
     # substitute the dict for kwargs
     if (len(kwargs) == 0 and
@@ -415,21 +428,25 @@ def load_kwargs(*args, **kwargs):
             isinstance(args[0], dict)):
         kwargs = args[0]
 
-    # function : list of expected keys
-    handlers = {handle_scene: ('graph', 'geometry'),
-                handle_trimesh_kwargs: ('vertices', 'faces'),
-                handle_trimesh_export: ('file_type', 'data')}
+    # (function, tuple of expected keys)
+    # order is important
+    handlers = (
+        (handle_scene, ('graph', 'geometry')),
+        (handle_mesh, ('vertices', 'faces')),
+        (handle_pointcloud, ('vertices',)),
+        (handle_export, ('file_type', 'data')))
+
+    # filter out keys with a value of None
+    kwargs = {k: v for k, v in kwargs.items() if v is not None}
 
     # loop through handler functions and expected key
-    handler = None
-    for func, expected in handlers.items():
+    for func, expected in handlers:
         if all(i in kwargs for i in expected):
             # all expected kwargs exist
             handler = func
             # exit the loop as we found one
             break
-
-    if handler is None:
+    else:
         raise ValueError('unable to determine type!')
 
     return handler()
@@ -440,50 +457,53 @@ def parse_file_args(file_obj,
                     resolver=None,
                     **kwargs):
     """
-    Given a file_obj and a file_type try to turn them into a file-like
-    object and a lowercase string of file type.
+    Given a file_obj and a file_type try to magically convert
+    arguments to a file-like object and a lowercase string of
+    file type.
 
     Parameters
     -----------
-    file_obj:  str: if string represents a file path, returns
-                    -------------------------------------------
-                    file_obj:   an 'rb' opened file object of the path
-                    file_type:  the extension from the file path
+    file_obj : str
+      if string represents a file path, returns:
+        file_obj:   an 'rb' opened file object of the path
+        file_type:  the extension from the file path
 
-               str: if string is NOT a path, but has JSON-like special characters
-                    -------------------------------------------
-                    file_obj:   the same string passed as file_obj
-                    file_type:  set to 'json'
+     if string is NOT a path, but has JSON-like special characters:
+        file_obj:   the same string passed as file_obj
+        file_type:  set to 'json'
 
-               str: string is a valid URL
-                    -------------------------------------------
-                    file_obj: an open 'rb' file object with retrieved data
-                    file_type: from the extension
+     if string is a valid-looking URL
+        file_obj: an open 'rb' file object with retrieved data
+        file_type: from the extension
 
-               str: string is not an existing path or a JSON-like object
-                    -------------------------------------------
-                    ValueError will be raised as we can't do anything with input
+     if string is none of those:
+        raise ValueError as we can't do anything with input
 
-               file like object: we cannot grab information on file_type automatically
-                    -------------------------------------------
-                    ValueError will be raised if file_type is None
-                    file_obj:  same as input
-                    file_type: same as input
+     if file like object:
+        ValueError will be raised if file_type is None
+        file_obj:  same as input
+        file_type: same as input
 
-               other object: like a shapely.geometry.Polygon, etc:
-                    -------------------------------------------
-                    file_obj:  same as input
-                    file_type: if None initially, set to the class name
-                               (in lower case), otherwise passed through
+     if other object: like a shapely.geometry.Polygon, etc:
+        file_obj:  same as input
+        file_type: if None initially, set to the class name
+                    (in lower case), otherwise passed through
 
-    file_type: str, type of file and handled according to above
+    file_type : str
+         type of file and handled according to above
 
     Returns
     -----------
-    file_obj:  loadable object
-    file_type: str, lower case of the type of file (eg 'stl', 'dae', etc)
-    metadata:  dict, any metadata
-    opened:    bool, did we open the file or not
+    file_obj : file-like object
+      Contains data
+    file_type : str
+      Lower case of the type of file (eg 'stl', 'dae', etc)
+    metadata : dict
+      Any metadata gathered
+    opened : bool
+      Did we open the file or not
+    resolver : trimesh.visual.Resolver
+      Resolver to load other assets
     """
     metadata = {}
     opened = False
@@ -491,8 +511,12 @@ def parse_file_args(file_obj,
             isinstance(kwargs['metadata'], dict)):
         metadata.update(kwargs['metadata'])
 
+    if util.is_pathlib(file_obj):
+        # convert pathlib objects to string
+        file_obj = str(file_obj.absolute())
+
     if util.is_file(file_obj) and file_type is None:
-        raise ValueError('file_type must be set when passing file objects!')
+        raise ValueError('file_type must be set for file objects!')
     if util.is_string(file_obj):
         try:
             # os.path.isfile will return False incorrectly
@@ -551,24 +575,35 @@ def parse_file_args(file_obj,
     # if we still have no resolver try using file_obj name
     if (resolver is None and
         hasattr(file_obj, 'name') and
+        file_obj.name is not None and
             len(file_obj.name) > 0):
         resolver = visual.resolvers.FilePathResolver(file_obj.name)
 
     return file_obj, file_type, metadata, opened, resolver
 
 
+# loader functions for compressed extensions
 compressed_loaders = {'zip': load_compressed,
                       'tar.bz2': load_compressed,
                       'tar.gz': load_compressed}
+
+# map file_type to loader function
 mesh_loaders = {}
 # assimp has a lot of loaders, but they are all quite slow
-# so we load them first and replace them with native loaders if possible
+# load first and replace with native loaders where possible
 mesh_loaders.update(_assimp_loaders)
+mesh_loaders.update(_misc_loaders)
 mesh_loaders.update(_stl_loaders)
 mesh_loaders.update(_ctm_loaders)
-mesh_loaders.update(_misc_loaders)
 mesh_loaders.update(_ply_loaders)
 mesh_loaders.update(_xml_loaders)
 mesh_loaders.update(_obj_loaders)
+mesh_loaders.update(_off_loaders)
+mesh_loaders.update(_collada_loaders)
 mesh_loaders.update(_gltf_loaders)
 mesh_loaders.update(_three_loaders)
+mesh_loaders.update(_xyz_loaders)
+
+# collect loaders which return voxel types
+voxel_loaders = {}
+voxel_loaders.update(_binvox_loaders)
