@@ -6,29 +6,32 @@ from ..constants import log
 
 from distutils.spawn import find_executable
 
+# start the search with the user's PATH
 _search_path = os.environ['PATH']
+
+# add additional search locations on windows
 if platform.system() == 'Windows':
     # split existing path by delimiter
     _search_path = [i for i in _search_path.split(';') if len(i) > 0]
-    _search_path.append(os.path.normpath('C:\Program Files\OpenSCAD'))
-    _search_path.append(os.path.normpath('C:\Program Files (x86)\OpenSCAD'))
+    _search_path.append(os.path.normpath(r'C:\Program Files\OpenSCAD'))
+    _search_path.append(os.path.normpath(r'C:\Program Files (x86)\OpenSCAD'))
     _search_path = ';'.join(_search_path)
     log.debug('searching for scad in: %s', _search_path)
-
+# add mac-specific search locations
 if platform.system() == 'Darwin':
     _search_path = [i for i in _search_path.split(':') if len(i) > 0]
     _search_path.append('/Applications/OpenSCAD.app/Contents/MacOS')
     _search_path = ':'.join(_search_path)
     log.debug('searching for scad in: %s', _search_path)
-    log.warning('searching for scad in: %s', _search_path)
 
+# try to find the SCAD executable by name
 _scad_executable = find_executable('openscad', path=_search_path)
-if not _scad_executable:
+if _scad_executable is None:
     _scad_executable = find_executable('OpenSCAD', path=_search_path)
 exists = _scad_executable is not None
 
 
-def interface_scad(meshes, script):
+def interface_scad(meshes, script, debug=False, **kwargs):
     """
     A way to interface with openSCAD which is itself an interface
     to the CGAL CSG bindings.
@@ -44,17 +47,17 @@ def interface_scad(meshes, script):
     """
     if not exists:
         raise ValueError('No SCAD available!')
-    with MeshScript(meshes=meshes, script=script) as scad:
-        result = scad.run(_scad_executable + ' $script -o $mesh_post')
+    with MeshScript(meshes=meshes, script=script, debug=debug) as scad:
+        result = scad.run(_scad_executable + ' $SCRIPT -o $MESH_POST')
     return result
 
 
-def boolean(meshes, operation='difference'):
+def boolean(meshes, operation='difference', debug=False, **kwargs):
     """
     Run an operation on a set of meshes
     """
     script = operation + '(){'
     for i in range(len(meshes)):
-        script += 'import(\"$mesh_' + str(i) + '\");'
+        script += 'import(\"$MESH_' + str(i) + '\");'
     script += '}'
-    return interface_scad(meshes, script)
+    return interface_scad(meshes, script, debug=debug, **kwargs)
