@@ -332,7 +332,7 @@ def plane_lines(plane_origin,
 
     d = np.divide(t[valid], b[valid])
     intersection = endpoints[0][valid]
-    intersection += np.reshape(d, (-1, 1)) * line_dir[valid]
+    intersection = intersection + np.reshape(d, (-1, 1)) * line_dir[valid]
 
     return intersection, valid
 
@@ -340,9 +340,11 @@ def plane_lines(plane_origin,
 def planes_lines(plane_origins,
                  plane_normals,
                  line_origins,
-                 line_directions):
+                 line_directions,
+                 return_distance=False,
+                 return_denom=False):
     """
-    Given one line per plane, find the intersection points.
+    Given one line per plane find the intersection points.
 
     Parameters
     -----------
@@ -354,13 +356,21 @@ def planes_lines(plane_origins,
         Point at origin of each line
     line_directions : (n,3) float
         Direction vector of each line
+    return_distance : bool
+      Return distance from origin to point also
+    return_denom : bool
+      Return denominator, so you can check for small values
 
     Returns
     ----------
     on_plane : (n,3) float
-        Points on specified planes
+      Points on specified planes
     valid : (n,) bool
-        Did plane intersect line or not
+      Did plane intersect line or not
+    distance : (n,) float
+      [OPTIONAL] Distance from point
+    denom : (n,) float
+      [OPTIONAL] Denominator
     """
 
     # check input types
@@ -375,7 +385,7 @@ def planes_lines(plane_origins,
     projection_ori = util.diagonal_dot(origin_vectors, plane_normals)
     projection_dir = util.diagonal_dot(line_directions, plane_normals)
 
-    valid = np.abs(projection_dir) > tol.merge
+    valid = np.abs(projection_dir) > 1e-5
 
     distance = np.divide(projection_ori[valid],
                          projection_dir[valid])
@@ -383,7 +393,14 @@ def planes_lines(plane_origins,
     on_plane = line_directions[valid] * distance.reshape((-1, 1))
     on_plane += line_origins[valid]
 
-    return on_plane, valid
+    result = [on_plane, valid]
+
+    if return_distance:
+        result.append(distance)
+    if return_denom:
+        result.append(projection_dir)
+
+    return result
 
 
 def slice_faces_plane(vertices,
@@ -468,11 +485,29 @@ def slice_faces_plane(vertices,
     # If no faces to cut, the surface is not in contact with this plane.
     # Thus, return a mesh with only the inside faces
     if len(cut_faces_quad) + len(cut_faces_tri) == 0:
-        vert_counts = np.bincount(new_faces.flatten(), minlength=len(vertices))
-        unique_verts = vert_counts > 0
-        unique_inds = np.where(unique_verts)[0]
+
+        if len(new_faces) == 0:
+            # if no new faces at all return empty arrays
+            empty = (np.zeros((0, 3), dtype=np.float64),
+                     np.zeros((0, 3), dtype=np.int64))
+            return empty
+
+        try:
+            # count the number of occurrences of each value
+            counts = np.bincount(new_faces.flatten(), minlength=len(vertices))
+            unique_verts = counts > 0
+            unique_index = np.where(unique_verts)[0]
+        except TypeError:
+            # casting failed on 32 bit windows
+            log.error('casting failed!', exc_info=True)
+            # fall back to numpy unique
+            unique_index = np.unique(new_faces.flatten())
+            # generate a mask for cumsum
+            unique_verts = np.zeros(len(vertices), dtype=np.bool)
+            unique_verts[unique_index] = True
+
         unique_faces = (np.cumsum(unique_verts) - 1)[new_faces]
-        return vertices[unique_inds], unique_faces
+        return vertices[unique_index], unique_faces
 
     # Extract the intersections of each triangle's edges with the plane
     o = cut_triangles                               # origins
@@ -552,9 +587,9 @@ def slice_faces_plane(vertices,
 
     # find the unique indices in the new faces
     # using an integer- only unique function
-    unique, inverse = util.unique_bincount(new_faces.reshape(-1),
-                                           minlength=len(new_vertices),
-                                           return_inverse=True)
+    unique, inverse = grouping.unique_bincount(new_faces.reshape(-1),
+                                               minlength=len(new_vertices),
+                                               return_inverse=True)
 
     # use the unique indexes for our final vertex and faces
     final_vert = new_vertices[unique]
@@ -574,18 +609,22 @@ def slice_mesh_plane(mesh,
     Parameters
     ---------
     mesh : Trimesh object
-        Source mesh to slice
+      Source mesh to slice
     plane_normal : (3,) float
-        Normal vector of plane to intersect with mesh
+      Normal vector of plane to intersect with mesh
     plane_origin:  (3,) float
-        Point on plane to intersect with mesh
+      Point on plane to intersect with mesh
+    cap: bool
+      If True, cap the result with a triangulated polygon
+
     cached_dots : (n, 3) float
         If an external function has stored dot
         products pass them here to avoid recomputing
+
     Returns
     ----------
     new_mesh : Trimesh object
-        Sliced mesh
+      Sliced mesh
     """
     # check input for none
     if mesh is None:
