@@ -1,3 +1,4 @@
+import os
 import numpy as np
 import collections
 
@@ -13,6 +14,9 @@ from .. import bounds as bounds_module
 from ..exchange import gltf
 from ..parent import Geometry
 
+from . import cameras
+from . import lighting
+
 from .transforms import TransformForest
 
 
@@ -28,7 +32,10 @@ class Scene(Geometry):
                  geometry=None,
                  base_frame='world',
                  metadata={},
-                 graph=None):
+                 graph=None,
+                 camera=None,
+                 lights=None,
+                 camera_transform=None):
         """
         Create a new Scene object.
 
@@ -42,6 +49,12 @@ class Scene(Geometry):
           Any metadata about the scene
         graph : TransformForest or None
           A passed transform graph to use
+        camera : Camera or None
+          A passed camera to use
+        lights : [trimesh.scene.lighting.Light] or None
+          A passed lights to use
+        camera_transform : (4, 4) float or None
+          Camera transform in the base frame
         """
         # mesh name : Trimesh object
         self.geometry = collections.OrderedDict()
@@ -63,10 +76,21 @@ class Scene(Geometry):
             # if we've been passed a graph override the default
             self.graph = graph
 
-        self.camera = None
+        self.camera = camera
+        self.lights = lights
+        self.camera_transform = camera_transform
 
     def apply_transform(self, transform):
-        raise NotImplementedError
+        """
+        Apply a transform to every geometry in the scene.
+
+        Parameters
+        --------------
+        transform : (4, 4)
+          Homogeneous transformation matrix
+        """
+        for geometry in self.geometry.values():
+            geometry.apply_transform(transform)
 
     def add_geometry(self,
                      geometry,
@@ -103,8 +127,19 @@ class Scene(Geometry):
         # PointCloud objects will look like a sequence
         elif util.is_sequence(geometry):
             # if passed a sequence add all elements
-            for value in geometry:
-                self.add_geometry(value)
+            for i, value in enumerate(geometry):
+                if i == 0:
+                    node_name = self.add_geometry(
+                        geometry=value,
+                        node_name=node_name,
+                        geom_name=geom_name,
+                        parent_node_name=parent_node_name,
+                        transform=transform)
+                else:
+                    self.add_geometry(
+                        geometry=value,
+                        geom_name=geom_name,
+                        parent_node_name=node_name)
             return
 
         elif isinstance(geometry, dict):
@@ -112,6 +147,18 @@ class Scene(Geometry):
             for key, value in geometry.items():
                 self.add_geometry(value, geom_name=key)
             return
+        elif isinstance(geometry, Scene):
+            # concatenate current scene with passed scene
+            concat = self + geometry
+            # replace geometry in-place
+            self.geometry.clear()
+            self.geometry.update(concat.geometry)
+            # replace graph data with concatenated graph
+            self.graph.transforms = concat.graph.transforms
+            return
+        elif not hasattr(geometry, 'vertices'):
+            util.log.warning('unknown type ({}) added to scene!'.format(
+                type(geometry).__name__))
 
         # get or create a name to reference the geometry by
         if geom_name is not None:
@@ -151,6 +198,26 @@ class Scene(Geometry):
                           geometry_flags={'visible': True})
         return node_name
 
+    def delete_geometry(self, names):
+        """
+        Delete one more multiple geometries from the scene and also
+        remove any node in the transform graph which references it.
+
+        Parameters
+        --------------
+        name : hashable
+          Name that references self.geometry
+        """
+        # make sure we have a set we can check
+        if util.is_string(names):
+            names = [names]
+        names = set(names)
+
+        # remove the geometry reference from relevant nodes
+        self.graph.remove_geometries(names)
+        # remove the geometries from our geometry store
+        [self.geometry.pop(name, None) for name in names]
+
     def md5(self):
         """
         MD5 of scene which will change when meshes or
@@ -158,7 +225,8 @@ class Scene(Geometry):
 
         Returns
         --------
-        hashed: str, MD5 hash of scene
+        hashed : str
+          MD5 hash of scene
         """
         # start with transforms hash
         hashes = [self.graph.md5()]
@@ -231,13 +299,16 @@ class Scene(Geometry):
         corners_inst = []
         # (n, 3) float corners of each geometry
         corners_geom = {k: bounds_module.corners(v.bounds)
-                        for k, v in self.geometry.items()}
+                        for k, v in self.geometry.items()
+                        if v.bounds is not None}
+        if len(corners_geom) == 0:
+            return np.array([])
 
         for node_name in self.graph.nodes_geometry:
             # access the transform and geometry name from node
             transform, geometry_name = self.graph[node_name]
             # not all nodes have associated geometry
-            if geometry_name is None:
+            if geometry_name not in corners_geom:
                 continue
             # transform geometry corners into where
             # the instance of the geometry is located
@@ -257,9 +328,13 @@ class Scene(Geometry):
 
         Returns
         --------
-        bounds: (2,3) float points for min, max corner
+        bounds : (2, 3) float or None
+          Position of [min, max] bounding box
+          Returns None if no valid bounds exist
         """
         corners = self.bounds_corners
+        if len(corners) == 0:
+            return None
         bounds = np.array([corners.min(axis=0),
                            corners.max(axis=0)])
         return bounds
@@ -271,7 +346,8 @@ class Scene(Geometry):
 
         Returns
         ----------
-        extents: (3,) float, bounding box sides length
+        extents : (3,) float
+          Bounding box sides length
         """
         return np.diff(self.bounds, axis=0).reshape(-1)
 
@@ -282,7 +358,8 @@ class Scene(Geometry):
 
         Returns
         -----------
-        scale: float, the mean of the bounding box edge lengths
+        scale : float
+          The mean of the bounding box edge lengths
         """
         scale = (self.extents ** 2).sum() ** .5
         return scale
@@ -294,11 +371,33 @@ class Scene(Geometry):
 
         Returns
         --------
-        centroid: (3) float point for center of bounding box
+        centroid : (3) float
+          Point for center of bounding box
         """
         centroid = np.mean(self.bounds, axis=0)
         return centroid
 
+    @caching.cache_decorator
+    def area(self):
+        """
+        What is the summed area of every geometry which
+        has area.
+
+        Returns
+        ------------
+        area : float
+          Summed area of every instanced geometry
+        """
+        # get the area of every geometry that has an area property
+        areas = {n: g.area for n, g in self.geometry.items()
+                 if hasattr(g, 'area')}
+        # get the name of every geometry instance in the scene
+        geoms = [self.graph[n][1] for n in
+                 self.graph.nodes_geometry]
+        # sum the area for every instanced geometry
+        area = sum(areas[n] for n in geoms if n in geoms)
+        return area
+
     @caching.cache_decorator
     def triangles(self):
         """
@@ -307,7 +406,8 @@ class Scene(Geometry):
 
         Returns
         ----------
-        triangles: (n,3,3) float, triangles in space
+        triangles : (n, 3, 3) float
+          Triangles in space
         """
         triangles = collections.deque()
         triangles_node = collections.deque()
@@ -342,9 +442,9 @@ class Scene(Geometry):
         Returns
         ---------
         triangles_index : (len(self.triangles),)
-            Node name for each triangle
+          Node name for each triangle
         """
-        populate = self.triangles
+        populate = self.triangles  # NOQA
         return self._cache['triangles_node']
 
     @caching.cache_decorator
@@ -354,7 +454,8 @@ class Scene(Geometry):
 
         Returns
         ---------
-        identifiers: dict, identifier md5: key in self.geometry
+        identifiers : dict
+          {Identifier MD5: key in self.geometry}
         """
         identifiers = {mesh.identifier_md5: name
                        for name, mesh in self.geometry.items()}
@@ -365,13 +466,13 @@ class Scene(Geometry):
         """
         Return a sequence of node keys of identical meshes.
 
-        Will combine meshes duplicated by copying in space with different keys in
-        self.geometry, as well as meshes repeated by self.nodes.
+        Will include meshes with different geometry but identical
+        spatial hashes as well as meshes repeated by self.nodes.
 
         Returns
         -----------
-        duplicates: (m) sequence of keys to self.nodes that represent
-                     identical geometry
+        duplicates : (m) sequenc
+          Keys of self.nodes that represent identical geometry
         """
         # if there is no geometry we can have no duplicate nodes
         if len(self.geometry) == 0:
@@ -380,29 +481,50 @@ class Scene(Geometry):
         # geometry name : md5 of mesh
         mesh_hash = {k: int(m.identifier_md5, 16)
                      for k, m in self.geometry.items()}
-
         # the name of nodes in the scene graph with geometry
         node_names = np.array(self.graph.nodes_geometry)
         # the geometry names for each node in the same order
         node_geom = np.array([self.graph[i][1] for i in node_names])
-
         # the mesh md5 for each node in the same order
         node_hash = np.array([mesh_hash[v] for v in node_geom])
-
         # indexes of identical hashes
         node_groups = grouping.group(node_hash)
-
-        # sequence of node names, where each sublist has identical geometry
-        duplicates = [np.sort(node_names[g]).tolist() for g in node_groups]
+        # sequence of node names where each
+        # sublist has identical geometry
+        duplicates = [np.sort(node_names[g]).tolist()
+                      for g in node_groups]
         return duplicates
 
+    def deduplicated(self):
+        """
+        Return a new scene where each unique geometry is only
+        included once and transforms are discarded.
+
+        Returns
+        -------------
+        dedupe : Scene
+          One copy of each unique geometry from scene
+        """
+        # collect geometry
+        geometry = {}
+        # loop through groups of identical nodes
+        for group in self.duplicate_nodes:
+            # get the name of the geometry
+            name = self.graph[group[0]][1]
+            # collect our unique collection of geometry
+            geometry[name] = self.geometry[name]
+
+        return Scene(geometry)
+
     def set_camera(self,
                    angles=None,
                    distance=None,
                    center=None,
-                   camera=None):
+                   resolution=None,
+                   fov=None):
         """
-        Add a transform to self.graph for 'camera'
+        Create a camera object for self.camera, and add
+        a transform to self.graph for it.
 
         If arguments are not passed sane defaults will be figured
         out which show the mesh roughly centered.
@@ -418,52 +540,159 @@ class Scene(Geometry):
         camera : Camera object
           Object that stores camera parameters
         """
-        # passed camera object
-        if camera is not None:
-            self.graph.update(frame_from='camera',
-                              frame_to=self.graph.base_frame,
-                              matrix=camera.transform)
-            self.camera = camera
-            return
+
+        if fov is None:
+            fov = np.array([60, 45])
 
         # if no geometry nothing to set camera to
         if len(self.geometry) == 0:
-            return
-
-        # use centroid if no center passed
-        if center is None:
-            center = self.centroid
-
-        # use scene AABB to set standoff distance
-        if distance is None:
-            # for a 60.0 degree horizontal FOV
-            distance = ((self.extents.max() / 2) /
-                        np.tan(np.radians(60.0) / 2.0))
-
+            self._camera = cameras.Camera(fov=fov)
+            self.graph[self._camera.name] = None
+            return self._camera
         # set with no rotation by default
         if angles is None:
             angles = np.zeros(3)
 
-        translation = np.eye(4)
-        translation[0:3, 3] = center
-        # offset by a distance set by the model size
-        # the FOV is set for the Y axis, we multiply by a lightly
-        # padded aspect ratio to make sure the model is in view
-        translation[2][3] += distance * 1.35
+        rotation = transformations.euler_matrix(*angles)
+        transform = cameras.look_at(
+            self.bounds_corners,
+            fov=fov,
+            rotation=rotation,
+            distance=distance,
+            center=center)
 
-        transform = np.dot(transformations.rotation_matrix(
-            angles[0],
-            [1, 0, 0],
-            point=center),
-            transformations.rotation_matrix(
-            angles[1],
-            [0, 1, 0],
-            point=center))
-        transform = np.dot(transform, translation)
+        if hasattr(self, '_camera') and self._camera is not None:
+            self._camera.fov = fov
+            if resolution is not None:
+                self._camera.resolution = resolution
+        else:
+            # create a new camera object
+            self._camera = cameras.Camera(fov=fov, resolution=resolution)
 
-        self.graph.update(frame_from='camera',
-                          frame_to=self.graph.base_frame,
-                          matrix=transform)
+        self.graph[self._camera.name] = transform
+
+        return self._camera
+
+    @property
+    def camera_transform(self):
+        """
+        Get camera transform in the base frame
+
+        Returns
+        -------
+        camera_transform : (4, 4) float
+          Camera transform in the base frame
+        """
+        return self.graph[self.camera.name][0]
+
+    def camera_rays(self):
+        """
+        Calculate the trimesh.scene.Camera origin and ray
+        direction vectors. Returns one ray per pixel as set
+        in camera.resolution
+
+        Returns
+        --------------
+        origins: (n, 3) float
+          Ray origins in space
+        vectors: (n, 3) float
+          Ray direction unit vectors in world coordinates
+        pixels : (n, 2) int
+          Which pixel does each ray correspond to in an image
+        """
+        vectors, pixels = self.camera.to_rays()
+        transform = self.camera_transform
+
+        # apply the rotation to the direction vectors
+        vectors = transformations.transform_points(
+            vectors,
+            transform,
+            translate=False)
+        # camera origin is single point so extract from transform
+        origins = (np.ones_like(vectors) *
+                   transformations.translation_from_matrix(
+                       transform))
+        return origins, vectors, pixels
+
+    @camera_transform.setter
+    def camera_transform(self, camera_transform):
+        """
+        Set the camera transform in the base frame
+
+        Parameters
+        ----------
+        camera_transform : (4, 4) float
+          Camera transform in the base frame
+        """
+        if camera_transform is None:
+            return
+        self.graph[self.camera.name] = camera_transform
+
+    @property
+    def camera(self):
+        """
+        Get the single camera for the scene. If not manually
+        set one will abe automatically generated.
+
+        Returns
+        ----------
+        camera : trimesh.scene.Camera
+          Camera object defined for the scene
+        """
+        # no camera set for the scene yet
+        if not hasattr(self, '_camera') or self._camera is None:
+            # will create a camera with everything in view
+            return self.set_camera()
+        assert self._camera is not None
+
+        return self._camera
+
+    @camera.setter
+    def camera(self, camera):
+        """
+        Set a camera object for the Scene.
+
+        Parameters
+        -----------
+        camera : trimesh.scene.Camera
+          Camera object for the scene
+        """
+        if camera is None:
+            return
+        self._camera = camera
+
+    @property
+    def lights(self):
+        """
+        Get a list of the lights in the scene. If nothing is
+        set it will generate some automatically.
+
+        Returns
+        -------------
+        lights : [trimesh.scene.lighting.Light]
+          Lights in the scene.
+        """
+        if not hasattr(self, '_lights') or self._lights is None:
+            # do some automatic lighting
+            lights, transforms = lighting.autolight(self)
+            # assign the transforms to the scene graph
+            for L, T in zip(lights, transforms):
+                self.graph[L.name] = T
+            # set the lights
+            self._lights = lights
+        return self._lights
+
+    @lights.setter
+    def lights(self, lights):
+        """
+        Assign a list of light objects to the scene
+
+        Parameters
+        --------------
+        lights : [trimesh.scene.lighting.Light]
+          Lights in the scene.
+        """
+        self._lights = lights
 
     def rezero(self):
         """
@@ -488,23 +717,35 @@ class Scene(Geometry):
                           matrix=matrix)
         self.graph.base_frame = new_base
 
-    def dump(self):
+    def dump(self, concatenate=False):
         """
-        Append all meshes in scene to a list of meshes.
+        Append all meshes in scene freezing transforms.
+
+        Parameters
+        ------------
+        concatenate : bool
+          If True, concatenate results into single mesh
 
         Returns
         ----------
-        dumped: (n,) list, of Trimesh objects transformed to their
-                           location the scene.graph
+        dumped : (n,) Trimesh or Trimesh
+          Trimesh objects transformed to their
+          location the scene.graph
         """
-        result = collections.deque()
-
+        result = []
         for node_name in self.graph.nodes_geometry:
             transform, geometry_name = self.graph[node_name]
-
+            # get a copy of the geometry
             current = self.geometry[geometry_name].copy()
+            # move the geometry vertices into the requested frame
             current.apply_transform(transform)
+            current.metadata['name'] = node_name
+            # save to our list of meshes
             result.append(current)
+
+        if concatenate:
+            return util.concatenate(result)
+
         return np.array(result)
 
     @caching.cache_decorator
@@ -520,77 +761,88 @@ class Scene(Geometry):
         hull = convex.convex_hull(points)
         return hull
 
-    def export(self, file_type=None):
+    def export(self,
+               file_obj=None,
+               file_type=None,
+               **kwargs):
         """
         Export a snapshot of the current scene.
 
         Parameters
         ----------
-        file_type: what encoding to use for meshes
-                   ie: dict, dict64, stl
+        file_obj : str, file-like, or None
+          File object to export to
+        file_type : str or None
+          What encoding to use for meshes
+          IE: dict, dict64, stl
 
         Returns
         ----------
-        export: dict with keys:
-                meshes: list of meshes, encoded as per file_type
-                transforms: edge list of transforms, eg:
-                             ((u, v, {'matrix' : np.eye(4)}))
+        export : bytes
+          Only returned if file_obj is None
         """
 
-        file_type = str(file_type).strip().lower()
-        if file_type == 'gltf':
-            return gltf.export_gltf(self)
-        elif file_type == 'glb':
-            return gltf.export_glb(self)
-
-        export = {'graph': self.graph.to_edgelist(),
-                  'geometry': {},
-                  'scene_cache': {'bounds': self.bounds.tolist(),
-                                  'extents': self.extents.tolist(),
-                                  'centroid': self.centroid.tolist(),
-                                  'scale': self.scale}}
-
+        # if we weren't passed a file type extract from file_obj
         if file_type is None:
-            file_type = {'Trimesh': 'ply',
-                         'Path2D': 'dxf'}
-
-        # if the mesh has an export method use it
-        # otherwise put the mesh
-        # itself into the export object
-        for geometry_name, geometry in self.geometry.items():
-            if hasattr(geometry, 'export'):
-                if isinstance(file_type, dict):
-                    # case where we have export types that are different
-                    # for different classes of objects.
-                    for query_class, query_format in file_type.items():
-                        if util.is_instance_named(geometry, query_class):
-                            export_type = query_format
-                            break
-                else:
-                    # if file_type is not a dict, try to export everything in the
-                    # the scene as that value like 'ply'
-                    export_type = file_type
-                exported = {'data': geometry.export(file_type=export_type),
-                            'file_type': export_type}
-                export['geometry'][geometry_name] = exported
+            if util.is_string(file_obj):
+                file_type = str(file_obj).split('.')[-1]
             else:
-                # case where mesh object doesn't have exporter
-                # might be that someone replaced the mesh with a URL
-                export['geometry'][geometry_name] = geometry
-        return export
+                raise ValueError('file_type not specified!')
 
-    def save_image(self, resolution=(1024, 768), **kwargs):
+        # always remove whitepace and leading characters
+        file_type = file_type.strip().lower().lstrip('.')
+
+        # now handle our different scene export types
+        if file_type == 'gltf':
+            data = gltf.export_gltf(self, **kwargs)
+        elif file_type == 'glb':
+            data = gltf.export_glb(self, **kwargs)
+        elif file_type == 'dict':
+            from ..exchange.export import scene_to_dict
+            data = scene_to_dict(self)
+        elif file_type == 'obj':
+            from ..exchange.obj import export_obj
+            data = export_obj(self)
+        elif file_type == 'dict64':
+            from ..exchange.export import scene_to_dict
+            data = scene_to_dict(self, use_base64=True)
+        else:
+            raise ValueError(
+                'unsupported export format: {}'.format(file_type))
+
+        # now write the data or return bytes of result
+        if hasattr(file_obj, 'write'):
+            # if it's just a regular file object
+            file_obj.write(data)
+        elif util.is_string(file_obj):
+            # assume strings are file paths
+            file_path = os.path.expanduser(
+                os.path.abspath(file_obj))
+            if util.is_string(data):
+                mode = 'w'
+            else:
+                mode = 'wb'
+            with open(file_path, mode) as f:
+                f.write(data)
+        else:
+            # no writeable file object so return data
+            return data
+
+    def save_image(self, resolution=None, **kwargs):
         """
         Get a PNG image of a scene.
 
         Parameters
         -----------
-        resolution: (2,) int, resolution to render image
-        **kwargs:  passed to SceneViewer constructor
+        resolution : (2,) int
+          Resolution to render image
+        **kwargs
+          Passed to SceneViewer constructor
 
         Returns
         -----------
-        png: bytes, render of scene in PNG form
+        png : bytes
+          Render of scene as a PNG
         """
         from ..viewer import render_scene
         png = render_scene(scene=self,
@@ -791,9 +1043,17 @@ class Scene(Geometry):
         # use the geometries copy method to
         # allow them to handle references to unpickle-able objects
         geometry = {n: g.copy() for n, g in self.geometry.items()}
+
+        if not hasattr(self, '_camera') or self._camera is None:
+            # if no camera set don't include it
+            camera = None
+        else:
+            # otherwise get a copy of the camera
+            camera = self.camera.copy()
         # create a new scene with copied geometry and graph
         copied = Scene(geometry=geometry,
-                       graph=self.graph.copy())
+                       graph=self.graph.copy(),
+                       camera=camera)
         return copied
 
     def show(self, viewer=None, **kwargs):
@@ -802,18 +1062,21 @@ class Scene(Geometry):
 
         Parameters
         -----------
-        viewer: str 'gl':       open a pyglet window
-                str,'notebook': return ipython.display.HTML
-                None: automatically pick based on whether or not
-                          we are in an ipython notebook
-        smooth : bool
-          Turn on or off automatic smooth shading
+        viewer: str
+          What kind of viewer to open, including
+          'gl' to open a pyglet window, 'notebook'
+          for a jupyter notebook or None
+        kwargs : dict
+          Includes `smooth`, which will turn
+          on or off automatic smooth shading
         """
 
         if viewer is None:
             # check to see if we are in a notebook or not
             from ..viewer import in_notebook
-            viewer = ['gl', 'notebook'][int(in_notebook())]
+            viewer = 'gl'
+            if in_notebook():
+                viewer = 'notebook'
 
         if viewer == 'gl':
             # this imports pyglet, and will raise an ImportError
@@ -874,8 +1137,7 @@ def split_scene(geometry):
                      metadata=metadata)
 
     # a single geometry so we are going to split
-    split = collections.deque()
-
+    split = []
     metadata = {}
     for g in util.make_sequence(geometry):
         split.extend(g.split())
